@@ -23,9 +23,11 @@
 #include "links_acl.h"
 
 int _link_updown(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
-		 unsigned int enabled, unsigned int connected)
+		 unsigned int enabled, unsigned int connected, unsigned int lock_stats)
 {
-	struct knet_link *link = &knet_h->host_index[host_id]->link[link_id];
+	struct knet_host *host = knet_h->host_index[host_id];
+	struct knet_link *link = &host->link[link_id];
+	int savederrno = 0;
 
 	if ((link->status.enabled == enabled) &&
 	    (link->status.connected == connected))
@@ -40,6 +42,16 @@ int _link_updown(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
 	    (!link->status.connected))
 		link->status.dynconnected = 0;
 
+	if (lock_stats) {
+		savederrno = pthread_mutex_lock(&link->link_stats_mutex);
+		if (savederrno) {
+			log_err(knet_h, KNET_SUB_LINK, "Unable to get stats mutex lock for host %u link %u: %s",
+				host_id, link_id, strerror(savederrno));
+			errno = savederrno;
+			return -1;
+		}
+	}
+
 	if (connected) {
 		time(&link->status.stats.last_up_times[link->status.stats.last_up_time_index]);
 		link->status.stats.up_count++;
@@ -53,6 +65,10 @@ int _link_updown(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
 			link->status.stats.last_down_time_index = 0;
 		}
 	}
+
+	if (lock_stats) {
+		pthread_mutex_unlock(&link->link_stats_mutex);
+	}
 	return 0;
 }
 
@@ -230,6 +246,13 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	link->latency_cur_samples = 0;
 	link->flags = flags;
 
+	savederrno = pthread_mutex_init(&link->link_stats_mutex, NULL);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_LINK, "Unable to initialize link stats mutex: %s", strerror(savederrno));
+		err = -1;
+		goto exit_unlock;
+	}
+
 	if (transport_link_set_config(knet_h, link, transport) < 0) {
 		savederrno = errno;
 		err = -1;
@@ -487,6 +510,8 @@ int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 		check_rmall(knet_h, sock, transport);
 	}
 
+	pthread_mutex_destroy(&link->link_stats_mutex);
+
 	memset(link, 0, sizeof(struct knet_link));
 	link->link_id = link_id;
 
@@ -560,7 +585,7 @@ int knet_link_set_enable(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		goto exit_unlock;
 	}
 
-	err = _link_updown(knet_h, host_id, link_id, enabled, link->status.connected);
+	err = _link_updown(knet_h, host_id, link_id, enabled, link->status.connected, 0);
 	savederrno = errno;
 
 	if (enabled) {
@@ -1102,7 +1127,7 @@ int knet_link_get_status(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		return -1;
 	}
 
-	savederrno = get_global_wrlock(knet_h);
+	savederrno = pthread_rwlock_rdlock(&knet_h->global_rwlock);
 	if (savederrno) {
 		log_err(knet_h, KNET_SUB_LINK, "Unable to get read lock: %s",
 			strerror(savederrno));
@@ -1129,8 +1154,18 @@ int knet_link_get_status(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		goto exit_unlock;
 	}
 
+	savederrno = pthread_mutex_lock(&link->link_stats_mutex);
+	if (savederrno) {
+		log_err(knet_h, KNET_SUB_LINK, "Unable to get stats mutex lock for host %u link %u: %s",
+			host_id, link_id, strerror(savederrno));
+		err = -1;
+		goto exit_unlock;
+	}
+
 	memmove(status, &link->status, struct_size);
 
+	pthread_mutex_unlock(&link->link_stats_mutex);
+
 	/* Calculate totals - no point in doing this on-the-fly */
 	status->stats.rx_total_packets =
 		status->stats.rx_data_packets +
