@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2010-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2010-2020 Red Hat, Inc.  All rights reserved.
  *
  * Author: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under LGPL-2.0+
  */
 
 #include "config.h"
@@ -20,7 +20,7 @@
 #include "logging.h"
 #include "threads_common.h"
 
-static struct pretty_names subsystem_names[] =
+static struct pretty_names subsystem_names[KNET_MAX_SUBSYSTEMS] =
 {
 	{ "common", KNET_SUB_COMMON },
 	{ "handle", KNET_SUB_HANDLE },
@@ -47,6 +47,7 @@ static struct pretty_names subsystem_names[] =
 	{ "lzo2comp", KNET_SUB_LZO2COMP },
 	{ "lzmacomp", KNET_SUB_LZMACOMP },
 	{ "bzip2comp", KNET_SUB_BZIP2COMP },
+	{ "zstdcomp", KNET_SUB_ZSTDCOMP },
 	{ "unknown", KNET_SUB_UNKNOWN }		/* unknown MUST always be last in this array */
 };
 
@@ -98,7 +99,7 @@ static int is_valid_subsystem(uint8_t subsystem)
 	return -1;
 }
 
-static struct pretty_names loglevel_names[] =
+static struct pretty_names loglevel_names[KNET_LOG_DEBUG + 1] =
 {
 	{ "ERROR", KNET_LOG_ERR },
 	{ "WARNING", KNET_LOG_WARN },
