@@ -435,18 +435,21 @@ int knet_handle_stop(knet_handle_t knet_h)
 	return 0;
 }
 
-static int _make_local_sockaddr(struct sockaddr_storage *lo, uint16_t offset, int family)
+static int _make_local_sockaddr(struct sockaddr_storage *lo, int offset, int family)
 {
-	uint32_t port;
+	in_port_t port;
 	char portstr[32];
 
-	/* Use the pid if we can. but makes sure its in a sensible range */
-	port = (uint32_t)getpid() + offset;
-	if (port < 1024) {
-		port += 1024;
-	}
-	if (port > 65536) {
-		port = port & 0xFFFF;
+	if (offset < 0) {
+		/*
+		 * api_knet_link_set_config needs to access the API directly, but
+		 * it does not send any traffic, so it´s safe to ask the kernel
+		 * for a random port.
+		 */
+		port = 0;
+	} else {
+		/* Use the pid if we can. but makes sure its in a sensible range */
+		port = (getpid() + offset) % (65536-1024) + 1024;
 	}
 	sprintf(portstr, "%u", port);
 	memset(lo, 0, sizeof(struct sockaddr_storage));
@@ -458,16 +461,61 @@ static int _make_local_sockaddr(struct sockaddr_storage *lo, uint16_t offset, in
 	return knet_strtoaddr("127.0.0.1", portstr, lo, sizeof(struct sockaddr_storage));
 }
 
-int make_local_sockaddr(struct sockaddr_storage *lo, uint16_t offset)
+int make_local_sockaddr(struct sockaddr_storage *lo, int offset)
 {
 	return _make_local_sockaddr(lo, offset, AF_INET);
 }
 
-int make_local_sockaddr6(struct sockaddr_storage *lo, uint16_t offset)
+int make_local_sockaddr6(struct sockaddr_storage *lo, int offset)
 {
 	return _make_local_sockaddr(lo, offset, AF_INET6);
 }
 
+int _knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+			  uint8_t transport, uint64_t flags, int family, int dynamic,
+			  struct sockaddr_storage *lo)
+{
+	int err = 0, savederrno = 0;
+	uint32_t port;
+	char portstr[32];
+
+	for (port = 1025; port < 65536; port++) {
+		sprintf(portstr, "%u", port);
+		memset(lo, 0, sizeof(struct sockaddr_storage));
+		if (family == AF_INET6) {
+			err = knet_strtoaddr("::1", portstr, lo, sizeof(struct sockaddr_storage));
+		} else {
+			err = knet_strtoaddr("127.0.0.1", portstr, lo, sizeof(struct sockaddr_storage));
+		}
+		if (err < 0) {
+			printf("Unable to convert loopback to sockaddr: %s\n", strerror(errno));
+			goto out;
+		}
+		errno = 0;
+		if (dynamic) {
+			err = knet_link_set_config(knet_h, host_id, link_id, transport, lo, NULL, flags);
+		} else {
+			err = knet_link_set_config(knet_h, host_id, link_id, transport, lo, lo, flags);
+		}
+		savederrno = errno;
+		if ((err < 0)  && (savederrno != EADDRINUSE)) {
+			printf("Unable to configure link: %s\n", strerror(savederrno));
+			goto out;
+		}
+		if (!err) {
+			printf("Using port %u\n", port);
+			goto out;
+		}
+	}
+
+	if (err) {
+		printf("No more ports available\n");
+	}
+out:
+	errno = savederrno;
+	return err;
+}
+
 void test_sleep(knet_handle_t knet_h, int seconds)
 {
 	if (is_memcheck() || is_helgrind()) {
