@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2018-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2018-2020 Red Hat, Inc.  All rights reserved.
  *
  * Author: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under GPL-2.0+
  */
 
 #include "config.h"
@@ -33,8 +33,12 @@ static int test(void)
 	}
 
 	device_name_tmp = nozzle_get_name_by_handle(nozzle);
-	if ((!device_name_tmp) && (errno != ENOENT)) {
-		printf("Unable to get name by handle\n");
+	if (!device_name_tmp) {
+		if (errno != ENOENT) {
+			printf("Unable to get name by handle\n");
+		} else {
+			printf("received incorrect errno!\n");
+		}
 		err = -1;
 		goto out_clean;
 	}
