@@ -3,7 +3,7 @@
  *
  * Author: Christine Caulfield <ccaulfie@redhat.com>
  *
- * This software licensed under GPL-2.0+, LGPL-2.0+
+ * This software licensed under GPL-2.0+
  */
 
 
@@ -34,6 +34,14 @@
 #define XML_DIR "../man/xml-knet"
 #define XML_FILE "libknet_8h.xml"
 
+/*
+ * This isn't a maximum size, it just defines how long a parameter
+ * type can get before we decide it's not worth lining everything up to.
+ * it's mainly to stop function pointer types (which can get VERY long because
+ * of all *their* parameters) making everything else 'line-up' over separate lines
+ */
+#define LINE_LENGTH 80
+
 static int print_ascii = 1;
 static int print_man = 0;
 static int print_params = 0;
@@ -107,12 +115,12 @@ static char *get_child(xmlNode *node, const char *tag)
 			refid = NULL;
 			for (child = this_node->children; child; child = child->next) {
 				if (child->content) {
-					strcat(buffer, (char *)child->content);
+					strncat(buffer, (char *)child->content, sizeof(buffer)-1);
 				}
 
 				if ((strcmp( (char*)child->name, "ref") == 0)) {
 					if (child->children->content) {
-						strcat(buffer,(char *)child->children->content);
+						strncat(buffer, (char *)child->children->content, sizeof(buffer)-1);
 					}
 					refid = get_attr(child, "refid");
 				}
@@ -167,7 +175,7 @@ static void get_param_info(xmlNode *cur_node, struct qb_list_head *list)
 			if (sub_tag->type == XML_ELEMENT_NODE && strcmp((char *)sub_tag->name, "parameternamelist") == 0) {
 				paramname = (char*)sub_tag->children->next->children->content;
 			}
-			if (sub_tag->type == XML_ELEMENT_NODE && strcmp((char *)sub_tag->name, "parameterdescription") == 0) {
+			if (paramname && sub_tag->type == XML_ELEMENT_NODE && strcmp((char *)sub_tag->name, "parameterdescription") == 0) {
 				paramdesc = (char*)sub_tag->children->next->children->content;
 
 				/* Add text to the param_map */
@@ -199,24 +207,24 @@ static char *get_text(xmlNode *cur_node, char **returntext)
 	for (this_tag = cur_node->children; this_tag; this_tag = this_tag->next) {
 		if (this_tag->type == XML_TEXT_NODE && strcmp((char *)this_tag->name, "text") == 0) {
 			if (not_all_whitespace((char*)this_tag->content)) {
-				strcat(buffer, (char*)this_tag->content);
-				strcat(buffer, "\n");
+				strncat(buffer, (char*)this_tag->content, sizeof(buffer)-1);
+				strncat(buffer, "\n", sizeof(buffer)-1);
 			}
 		}
 		if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "emphasis") == 0) {
 			if (print_man) {
-				strcat(buffer, "\\fB");
+				strncat(buffer, "\\fB", sizeof(buffer)-1);
 			}
-			strcat(buffer, (char*)this_tag->children->content);
+			strncat(buffer, (char*)this_tag->children->content, sizeof(buffer)-1);
 			if (print_man) {
-				strcat(buffer, "\\fR");
+				strncat(buffer, "\\fR", sizeof(buffer)-1);
 			}
 		}
 		if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "itemizedlist") == 0) {
 			for (sub_tag = this_tag->children; sub_tag; sub_tag = sub_tag->next) {
 				if (sub_tag->type == XML_ELEMENT_NODE && strcmp((char *)sub_tag->name, "listitem") == 0) {
-					strcat(buffer, (char*)sub_tag->children->children->content);
-					strcat(buffer, "\n");
+					strncat(buffer, (char*)sub_tag->children->children->content, sizeof(buffer)-1);
+					strncat(buffer, "\n", sizeof(buffer)-1);
 				}
 			}
 		}
@@ -231,6 +239,7 @@ static char *get_text(xmlNode *cur_node, char **returntext)
 			if (returntext && strcmp(kind, "return") == 0) {
 				*returntext = tmp;
 			}
+			free(kind);
 		}
 
 		if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "parameterlist") == 0) {
@@ -241,6 +250,7 @@ static char *get_text(xmlNode *cur_node, char **returntext)
 			if (strcmp(kind, "retval") == 0) {
 				get_param_info(this_tag, &retval_list);
 			}
+			free(kind);
 		}
 	}
 	return strdup(buffer);
@@ -332,19 +342,25 @@ static int read_structure_from_xml(char *refid, char *name)
 
 static void print_param(FILE *manfile, struct param_info *pi, int field_width, int bold, const char *delimiter)
 {
-	char asterisk = ' ';
+	char *asterisks = "  ";
 	char *type = pi->paramtype;
 
 	/* Reformat pointer params so they look nicer */
 	if (pi->paramtype[strlen(pi->paramtype)-1] == '*') {
-		asterisk='*';
+		asterisks=" *";
 		type = strdup(pi->paramtype);
 		type[strlen(type)-1] = '\0';
+
+		/* Cope with double pointers */
+		if (pi->paramtype[strlen(type)-1] == '*') {
+			asterisks="**";
+			type[strlen(type)-1] = '\0';
+		}
 	}
 
-	fprintf(manfile, "    %s%-*s%c%s\\fI%s\\fP%s\n",
+	fprintf(manfile, "    %s%-*s%s%s\\fI%s\\fP%s\n",
 		bold?"\\fB":"", field_width, type,
-		asterisk, bold?"\\fP":"", pi->paramname, delimiter);
+		asterisks, bold?"\\fP":"", pi->paramname, delimiter);
 
 	if (type != pi->paramtype) {
 		free(type);
@@ -400,17 +416,17 @@ char *get_texttree(int *type, xmlNode *cur_node, char **returntext)
 
 		if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "para") == 0) {
 			tmp = get_text(this_tag, returntext);
-			strcat(buffer, tmp);
-			strcat(buffer, "\n");
+			strncat(buffer, tmp, sizeof(buffer)-1);
+			strncat(buffer, "\n", sizeof(buffer)-1);
 			free(tmp);
 		}
 	}
 
 	if (buffer[0]) {
-		tmp = strdup(buffer);
+		return strdup(buffer);
+	} else {
+		return NULL;
 	}
-
-	return tmp;
 }
 
 /* The text output is VERY basic and just a check that it's working really */
@@ -504,7 +520,8 @@ static void print_manpage(char *name, char *def, char *brief, char *args, char *
 	qb_list_for_each(iter, &params_list) {
 		pi = qb_list_entry(iter, struct param_info, list);
 
-		if (strlen(pi->paramtype) > max_param_type_len) {
+		if ((strlen(pi->paramtype) < LINE_LENGTH) &&
+		    (strlen(pi->paramtype) > max_param_type_len)) {
 			max_param_type_len = strlen(pi->paramtype);
 		}
 		if (strlen(pi->paramname) > max_param_name_len) {
@@ -559,11 +576,6 @@ static void print_manpage(char *name, char *def, char *brief, char *args, char *
 
 		map_iter = qb_map_iter_create(used_structures_map);
 		for (p = qb_map_iter_next(map_iter, &data); p; p = qb_map_iter_next(map_iter, &data)) {
-			fprintf(manfile, ".SS \"\"\n");
-			fprintf(manfile, ".PP\n");
-			fprintf(manfile, ".sp\n");
-			fprintf(manfile, ".sp\n");
-			fprintf(manfile, ".RS\n");
 			fprintf(manfile, ".nf\n");
 			fprintf(manfile, "\\fB\n");
 
@@ -660,6 +672,7 @@ static void collect_functions(xmlNode *cur_node, void *arg)
 				num_functions++;
 			}
 		}
+		free(kind);
 	}
 }
 
@@ -682,20 +695,23 @@ static void collect_enums(xmlNode *cur_node, void *arg)
 			for (this_tag = cur_node->children; this_tag; this_tag = this_tag->next) {
 				if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "name") == 0) {
 					name = strdup((char *)this_tag->children->content);
+					break;
 				}
 			}
 
-			si = malloc(sizeof(struct struct_info));
-			if (si) {
-				si->kind = STRUCTINFO_ENUM;
-				qb_list_init(&si->params_list);
-				si->structname = strdup(name);
-				traverse_node(cur_node, "enumvalue", read_struct, si);
-				qb_map_put(structures_map, refid, si);
+			if (name) {
+				si = malloc(sizeof(struct struct_info));
+				if (si) {
+					si->kind = STRUCTINFO_ENUM;
+					qb_list_init(&si->params_list);
+					si->structname = strdup(name);
+					traverse_node(cur_node, "enumvalue", read_struct, si);
+					qb_map_put(structures_map, refid, si);
+				}
+				free(name);
 			}
-
 		}
-
+		free(kind);
 	}
 }
 
@@ -722,14 +738,14 @@ static void traverse_members(xmlNode *cur_node, void *arg)
 			if (!this_tag->children || !this_tag->children->content)
 				continue;
 
-			if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "definition") == 0)
+			if (!def && this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "definition") == 0)
 				def = strdup((char *)this_tag->children->content);
-			if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "argsstring") == 0)
+			if (!args && this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "argsstring") == 0)
 				args = strdup((char *)this_tag->children->content);
-			if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "name") == 0)
+			if (!name && this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "name") == 0)
 				name = strdup((char *)this_tag->children->content);
 
-			if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "briefdescription") == 0) {
+			if (!brief && this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "briefdescription") == 0) {
 				brief = get_texttree(&type, this_tag, &returntext);
 				if (brief) {
 					/*
@@ -739,7 +755,7 @@ static void traverse_members(xmlNode *cur_node, void *arg)
 					brief[strlen(brief) - 3] = '\0';
 				}
 			}
-			if (this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "detaileddescription") == 0) {
+			if (!detailed && this_tag->type == XML_ELEMENT_NODE && strcmp((char *)this_tag->name, "detaileddescription") == 0) {
 				detailed = get_texttree(&type, this_tag, &returntext);
 			}
 			/* Get all the params */
@@ -757,7 +773,6 @@ static void traverse_members(xmlNode *cur_node, void *arg)
 		}
 
 		if (kind && strcmp(kind, "function") == 0) {
-
 			/* Make sure function has a doxygen description */
 			if (!detailed) {
 				fprintf(stderr, "No doxygen description for function '%s' - please fix this\n", name);
@@ -776,7 +791,9 @@ static void traverse_members(xmlNode *cur_node, void *arg)
 		free(kind);
 		free(def);
 		free(args);
-//		free(name); /* don't free, it's in the map */
+		free(detailed);
+		free(brief);
+		free(name);
 	}
 }
 
